#include <iostream>
#include <string>
#include "libsqlite.hpp" //include sqlite header file
using namespace std;

// connect to DB
sqlite::sqlite db("timetableDat.db");

//Welcome user and get input function
int ask_user(){
    int answer;
    cout << "-----------------------\nPress [1] to VIEW all lecturers.\nPress [2] to VIEW all students.\nPress [3] to ADD a lecturer.\nPress [4] to ADD a student.\nPress [5] to ADD a subject.\nPress [6] to CHANGE a students subject.\nPress [7] to VIEW a student's lesson information.\n-----------------------\n[Press anything else to quit]\n-----------------------" << endl;
    cin >> answer;

    return answer;
}

//simple quit function
void quit(){
    cout << "----------\n|Goodbye!|\n----------" << endl;
}

//list lecturer columns
void list_lecturers(){
      //cursor always re-initialized in functions so that the same operation can be repeated
      auto cur = db.get_statement(); //initialize cursor1
      cur->set_sql("SELECT * FROM Lecturers;");
      cur->prepare();
      //loop through results
      while (cur->step()){
          cout << cur->get_text(1) << " | ID: " << cur->get_text(0) << " | Age: " << cur->get_text(2) << endl;
      }
      cout << "-----------------------" << endl;
}

//list all student columns
void list_students(){
      auto cur = db.get_statement(); //initialize cursor1
      cur->set_sql("SELECT * FROM Students;");
      cur->prepare();
      while (cur->step()){
          cout << cur -> get_text(1) << " | ID: " << cur->get_text(0) << " | Age: " << cur->get_text(2) << " | Subject: " << cur->get_text(3) << endl;
      } 
      cout << "-----------------------" << endl;
}

//add lecturer to db
void add_lecturer(){
      // get needed input form user
      string lec_name, lec_name2, lec_full_name, lec_id, lec_age;
      cout << "Please enter first name: " << endl;
      cin >> lec_name;
      cout << "Please enter last name: " << endl;
      cin >> lec_name2;
      lec_full_name = lec_name + " " + lec_name2;
      cout << "Please enter lecturer ID: " << endl;
      cin >> lec_id;
      cout << "Please enter age: " << endl;
      cin >> lec_age;

      auto cur = db.get_statement(); //initialize cursor1
      cur->set_sql("INSERT INTO Lecturers ( LID, Name, Age) VALUES (?, ?, ?);");
      cur->prepare();
      cur->bind(1, lec_id);
      cur->bind(2, lec_full_name);
      cur->bind(3, lec_age);
      cur->step();
      cout << "\n\nSUCCESS!\n\n" << endl;
}

//add student to db
void add_student(){
      // get needed input form user
      string stu_name, stu_name2, stu_full_name, stu_id, stu_age, stu_subject;
      cout << "Please enter first name: " << endl;
      cin >> stu_name;
      cout << "Please enter last name: " << endl;
      cin >> stu_name2;
      stu_full_name = stu_name + " " + stu_name2;
      cout << "Please enter student ID: " << endl;
      cin >> stu_id;
      cout << "Please enter age: " << endl;
      cin >> stu_age;
      cout << "Please enter Subject: " << endl;
      cin >> stu_subject;

      auto cur = db.get_statement(); //initialize cursor1
      cur->set_sql("INSERT INTO Students ( ID, Name, Age, Subject) VALUES (?, ?, ?, ?);");
      cur->prepare();
      cur->bind(1, stu_id);
      cur->bind(2, stu_full_name);
      cur->bind(3, stu_age);
      cur->bind(4, stu_subject);
      cur->step();
      cout << "\n\nSUCCESS!\n\n" << endl;
}

//change students subject choice in db
void change_subject(){
      string stu_id, new_choice;
      cout << "Please enter student ID: " << endl;
      cin >> stu_id;
      cout << "Please type one of the below subjects:" << endl;

      auto cur = db.get_statement(); //initialize cursor1
      // Show all Subjects (allow user to chose one)
      cur->set_sql("SELECT Subject FROM Modules;");
      cur->prepare();
      while (cur->step()){
          cout << cur -> get_text(0) << endl;
      }
      cin >> new_choice;

      auto cur2 = db.get_statement(); //initialize cursor 2
      cur2->set_sql("UPDATE Students SET Subject=? WHERE ID=?;");
      cur2->prepare();
      cur2->bind(1, new_choice);
      cur2->bind(2, stu_id);
      cur2->step();
      cout << "\n\nSUCCESS!\n\n" << endl;
}

//ability to add a subject
void add_subject(){
    string subj_name, mod1, mod2, mod3;
      cout << "Please enter the subject name: " << endl;
      cin >> subj_name;
      cout << "Please enter the name of module 1: " << endl;
      cin >> mod1;
      cout << "Please enter the name of module 2: " << endl;
      cin >> mod2;     
      cout << "Please enter the name of module 3: " << endl;
      cin >> mod3;

      auto cur = db.get_statement(); //initialize cursor1
      cur->set_sql("INSERT INTO Modules ( Subject, Module1, Module2, Module3) VALUES (?, ?, ?, ?);");
      cur->prepare();
      cur->bind(1, subj_name);
      cur->bind(2, mod1);
      cur->bind(3, mod2);
      cur->bind(4, mod3);
      cur->step();
      cout << "\n\nSUCCESS!\n\n" << endl;
}

void view_timetable(){
     const char separator    = ' ';
     const int textWidth     = 25;
  
      string tt_stu_id;
      cout << "Please enter students ID:" << endl;
      cin >> tt_stu_id;
      auto cur = db.get_statement(); //initialize cursor1
      cur->set_sql("SELECT c.CID, c.Module, c.Room, l.Name FROM Students s, Modules m, Classes c, Lecturers l WHERE (s.ID = ? AND s.Subject = m.Subject) AND ((m.Module1 = c.Module) OR (m.Module2 = c.Module) OR (m.Module3 = c.Module)) AND (l.LID = c.LID)");  
      cur->prepare();
      cur->bind(1, tt_stu_id);
      while (cur->step()){
          cout << "Class ID: " << cur->get_text(0) << " | Module Name: " << cur->get_text(1) << " | Room: " << cur->get_text(2) << " | Lecturer Name: Mr/Mrs " << cur->get_text(3) << endl;
      }
      cout << "-----------------------" << endl;
}

//ask if user wants to repeat the function
bool repeater(){
       bool again;
       char result;
       cout << "Would you like to do something else? [Y]/[N].\n-----------------------" << endl;
       cin >> result;
       if (result == 'Y' || result == 'y') {
           again = true;
       }
       else {
           again = false;
       }
    
       return again;
}

//main function
int main() {
    cout << "\n-----------------------\n|timetable admin panel|\n-----------------------" << endl;
    
    bool again = true;
    while(again == true){
        //take and chack user input
        int user_in = ask_user();        
        if (user_in == 1){
            list_lecturers();
            //run again?
            again = repeater();
        }
        else if (user_in == 2){
            list_students();
            //run again?
            again = repeater();
        }
        else if (user_in == 3){
            add_lecturer();
            //run again?
            again = repeater();
        }
        else if (user_in == 4){
            add_student();
            //run again?
            again = repeater();
        }
        else if (user_in == 5){
            add_subject();
            //run again?
            again = repeater();
        }
        else if (user_in == 6){
            change_subject();
            //run again?
            again = repeater();
        }
        else if (user_in == 7){
            view_timetable();
            //run again?
            again = repeater();
        }
        // ensure no ugly errors produced
        else {
            quit();
            again = false;
       }        
    }
}
//g++ --std=c++14 main.cpp -o main -lsqlite3
//./main